package com.tzld.piaoquan.recommend.framework.ranker;



import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.typesafe.config.Config;
import com.tzld.piaoquan.recommend.framework.common.item.RankerItem;
import com.tzld.piaoquan.recommend.framework.common.user.User;
import com.tzld.piaoquan.recommend.server.gen.recommend.RecommendRequest;
import org.apache.commons.lang.exception.ExceptionUtils;

import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

public class RankPipeline {
    private List<AbstractRanker> rankers = Lists.newArrayList();
    private Map<String, Integer> boostMap;

    public RankPipeline(List<AbstractRanker> rankers) {
        this.rankers = rankers;
    }

    public void setBoostMap(Map<String, Integer> boostMap) {
        this.boostMap = boostMap;
    }

    public static void boostQueues(List<RankerItem> rankerItems, Map<String, Integer> boostMap) {
        if (boostMap == null || boostMap.isEmpty()) {
            return ;
        }

        List<RankerItem> boostItems = Lists.newArrayList();
        Map<String, Integer> boostCounts = Maps.newConcurrentMap();
        boostCounts.putAll(boostMap);
        for (Iterator<RankerItem> itemIterator = rankerItems.iterator(); itemIterator.hasNext(); ) {
            RankerItem currentItem = itemIterator.next();
            if (boostCounts.containsKey(currentItem.getQueue())) {
                itemIterator.remove();
                currentItem.setScore((currentItem.getScore() + 1.0) * 100.0); // FIXME: boost priority
                boostItems.add(currentItem);
                int myQueueBoostCount = boostCounts.get(currentItem.getQueue()) - 1;
                if (myQueueBoostCount > 0) {
                    boostCounts.put(currentItem.getQueue(), myQueueBoostCount);
                } else {
                    boostCounts.remove(currentItem.getQueue());
                }
            }
        }
        rankerItems.addAll(0, boostItems);
    }

    public List<RankerItem> doRank(final RecommendRequest requestContext, final User user, final int requestIndex, List<RankerItem> rankerItems) {
        // initial rank by final score
        Collections.sort(rankerItems);

        for (AbstractRanker ranker : rankers) {
            ranker.rank(requestContext, user, requestIndex, rankerItems);
        }

        // boost
        if (boostMap != null && !boostMap.isEmpty()) {
            boostQueues(rankerItems, boostMap);
        }

        return rankerItems;
    }

    public static List<AbstractRanker> constructRankers(Config rankersConfig) {
        List<AbstractRanker> rankers = Lists.newArrayList();
        for (RankerConfig config : RankerConfig.loadRankerConfig(rankersConfig)) {
            try {
                AbstractRanker ranker = (AbstractRanker) Class.forName(config.getRankerClass()).getConstructor(config.getClass()).newInstance(config);
                rankers.add(ranker);
            } catch (Exception e) {
                //
            }
        }

        return rankers;
    }
}
