package com.tzld.piaoquan.recommend.framework.ranker;


import com.google.common.collect.Lists;
import com.typesafe.config.Config;
import com.typesafe.config.ConfigObject;
import com.typesafe.config.ConfigValue;
import com.tzld.piaoquan.recommend.framework.utils.ConfigUtils;
import org.apache.commons.lang.exception.ExceptionUtils;

import java.util.Collections;
import java.util.List;

public class RankerConfig implements Comparable<RankerConfig> {

    private String name;
    private String rankerClass;

    private int rankPriority;
    private boolean enableSwitch;

    private String experimentKey;

    private Config paramConfig;

    public RankerConfig(String name, String rankerClass, int rankPriority, boolean enableSwitch, String experimentKey, Config paramConfig) {
        this.name = name;
        this.rankerClass = rankerClass;
        this.rankPriority = rankPriority;
        this.enableSwitch = enableSwitch;
        this.experimentKey = experimentKey;
        this.paramConfig = paramConfig;
    }

    public String getName() {
        return name;
    }

    public String getRankerClass() {
        return rankerClass;
    }

    public int getPriority() {
        return rankPriority;
    }

    public boolean isEnable() {
        return enableSwitch;
    }

    public String getExperimentKey() {
        return experimentKey;
    }

    public Config getParamConfig() {
        return paramConfig;
    }


    public static List<RankerConfig> loadRankerConfig(Config config) {
        List<RankerConfig> rankerConfigList = Lists.newArrayList();
        try {
            for (ConfigObject.Entry<String, ConfigValue> it : config.getConfig("ranker-config").root().entrySet()) {
                String name = it.getKey();
                Config entryConf = ((ConfigObject)it.getValue()).toConfig();
                int priority = ConfigUtils.loadOrDefault(entryConf, "ranker-priority", 100);
                boolean enableSwitch = ConfigUtils.loadOrDefault(entryConf, "enable-switch", true);
                String experimentKey = ConfigUtils.loadOrDefault(entryConf, "experiment-key", "");
                Config paramConfig = ConfigUtils.loadOrDefault(entryConf, "param-config", (Config) null);
                RankerConfig rankerConfig = new RankerConfig(name, entryConf.getString("ranker"), priority, enableSwitch, experimentKey, paramConfig);
                rankerConfigList.add(rankerConfig);
                //按照priority倒叙排序
                Collections.sort(rankerConfigList);
            }
        } catch (Exception e) {
            // log
        }

        return rankerConfigList;
    }

    /**
     * 降序排列
     * @param o
     * @return
     */
    @Override
    public int compareTo(RankerConfig o) {
        if (o == null) {
            return -1;
        }
        if (this.rankPriority == o.rankPriority) {
            return 0;
        }
        return this.rankPriority < o.rankPriority ? 1 : -1;
    }
}
