package com.tzld.piaoquan.recommend.framework.retrieve;

import com.typesafe.config.Config;
import com.typesafe.config.ConfigFactory;
import com.typesafe.config.ConfigObject;
import com.typesafe.config.ConfigValue;

import com.tzld.piaoquan.recommend.framework.retrieve.candidate.IndexDescription;
import com.tzld.piaoquan.recommend.framework.utils.ParamUtils;
import com.tzld.piaoquan.recommend.server.gen.recommend.RecommendRequest;
import java.util.HashMap;
import java.util.Map;

public class RetrieveConfig {
    private final int priority;
    private final int recallWeight;
    private final int bsWeight;
    private final int asProtectCount;
    private final int boostCount;

    private final boolean enable;
    private final String enableKey;
    private final Map<String, IndexDescription> childIndics;
    private final Config paramConfig;

    public RetrieveConfig(int priority,
                          int recallWeight, int bsWeight, int asProtectCount, int boostCount,
                          boolean enable, String enableKey,
                          Config indexDescConfig, Config paramConfig) {
        this.priority = priority;
        this.recallWeight = recallWeight;
        this.bsWeight = bsWeight;
        this.asProtectCount = asProtectCount;
        this.boostCount = boostCount;
        this.enable = enable;
        this.enableKey = enableKey;
        this.childIndics = parseChildIndics(indexDescConfig);
        this.paramConfig = paramConfig;
    }

    public int getRecallWeight() {
        return recallWeight;
    }

    public int getBsWeight() {
        return bsWeight;
    }

    public int getAsProtectCount() {
        return asProtectCount;
    }

    public int getBoostCount() {
        return boostCount;
    }

    public int getPriority() {
        return priority;
    }

    public boolean isEnable(final RecommendRequest requestData, final String layerPath) {
        return enable || ParamUtils.getExpParamValue(requestData, layerPath, getEnableKey()).equals("1");
    }

    public String getEnableKey() {
        return enableKey;
    }

    public Map<String, IndexDescription> getChildIndics() {
        return childIndics;
    }

    public Config getParamConfig() {
        return paramConfig;
    }

    @Override
    public String toString() {
        return "RetrieveConfig{" + "\'" +
                "priority=" + priority + "\'" +
                ", recall-weight=" + recallWeight + "\'" +
                ", bs-weight=" + bsWeight + "\'" +
                ", as-protect-count=" + asProtectCount + "\'" +
                ", boost-count=" + boostCount + "\'" +
                ", enable_key='" + enableKey + "\'" +
                ", paramConfig=" + paramConfig +
                '}';
    }

    public static RetrieveConfig parseRetrieveConfig(Config retrieveConfig) {
        int recallWeight = retrieveConfig.getInt("recall-weight");
        int bsWeight = retrieveConfig.getInt("bs-weight");
        int asProtectCount = retrieveConfig.getInt("as-protect-count");
        int boostCount = retrieveConfig.getInt("boost-count");
        //默认是0
        int priority = loadOptionIntConfig(retrieveConfig, "priority");
        //默认为true
        boolean enableSwitch = loadOptionBooleanConfig(retrieveConfig, "enable-switch");

        String enableKey = loadOptionStringConfig(retrieveConfig,"experiment-enable-key");
        Config paramConfig = loadOptionConfig(retrieveConfig, "param-config");
        Config indexConfig = loadOptionConfig(retrieveConfig, "children-index"); // FIXME: try load
        return new RetrieveConfig(priority,
                recallWeight, bsWeight, asProtectCount, boostCount,
                enableSwitch, enableKey, indexConfig, paramConfig);
    }

    private static Map<String, IndexDescription> parseChildIndics(Config indexDescConfig) {
        Map<String, IndexDescription> indexDescMap = new HashMap<String, IndexDescription>();
        ConfigObject confObj = indexDescConfig.root();
        for (ConfigObject.Entry<String, ConfigValue> it : confObj.entrySet()) {
            String myName = it.getKey();
            Config myConf = ((ConfigObject) it.getValue()).toConfig();
            IndexDescription index = IndexDescription.parseIndexDescription(myName, myConf);
            indexDescMap.put(myName, index);
        }
        return indexDescMap;
    }

    // FIXME: ConfigLoadUtils
    private static String loadOptionStringConfig(Config config, String path) {
        return config.hasPath(path) ? config.getString(path) : null;
    }

    private static Config loadOptionConfig(Config config, String path) {
        return config.hasPath(path) ? config.getConfig(path) : ConfigFactory.empty();
    }

    private static int loadOptionIntConfig(Config config, String path) {
        return config.hasPath(path) ? config.getInt(path) : 0;
    }

    /**
     * 默认为true
     * @param config
     * @param path
     * @return
     */
    private static boolean loadOptionBooleanConfig(Config config, String path) {
        return config.hasPath(path) ? config.getBoolean(path) : true;
    }
}
