package com.tzld.piaoquan.recommend.framework.retrieve;



import com.google.common.collect.Maps;
import com.typesafe.config.Config;
import com.typesafe.config.ConfigObject;
import com.typesafe.config.ConfigValue;
import com.tzld.piaoquan.recommend.framework.retrieve.candidate.Candidate;
import com.tzld.piaoquan.recommend.framework.common.user.User;
import com.tzld.piaoquan.recommend.framework.retrieve.candidate.IndexDescription;
import com.tzld.piaoquan.recommend.framework.utils.ConfigUtils;
import com.tzld.piaoquan.recommend.framework.utils.ParamUtils;
import com.tzld.piaoquan.recommend.server.gen.recommend.RecommendRequest;
import org.apache.commons.lang.exception.ExceptionUtils;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;


public class RetrievePipelineUtils {
    private static final Logger LOGGER = LoggerFactory.getLogger(RetrievePipelineUtils.class);

    public static RetrieverPipeline getRetrievePipeline(final Map<Integer, Config> retrieveConfigMap, final RecommendRequest requestData) {
        List<Config> expConfigList = new ArrayList<Config>();

        // TODO: replace with retrieve-layer-config-key

         Set<Integer> expIdSet = ParamUtils.getExpIdSet(requestData);
         for (Integer expId : expIdSet) {
             ConfigUtils.addConfig(expId, retrieveConfigMap, expConfigList);
        }

        Config retrieveBaseConfig = null;
        if (retrieveConfigMap.containsKey("base")) {
            retrieveBaseConfig = retrieveConfigMap.get("base");
        } else if (retrieveConfigMap.containsKey("0")) {
            retrieveBaseConfig = retrieveConfigMap.get("0");
        } else {
            // TODO: throw exception
        }
        return getRetrievePipeline(retrieveBaseConfig, expConfigList);
    }

    public static int sumTotalRecallCount(Map<String, Candidate> candidateMap) {
        int total = 0;
        for (Map.Entry<String, Candidate> candidateEntry : candidateMap.entrySet()) {
            total += candidateEntry.getValue().getMergeQueueNum();
        }
        return total;
    }

    public static int normalizeRetrieve(Map<String, Candidate> candidates, final int recallCount) {
        // TODO: normalize retrieve weight
        int initRecallCount = sumTotalRecallCount(candidates);

        if (initRecallCount <= 0) {
            return 0;
        }

        double normalizeRatio = recallCount * 1.0 / initRecallCount;
        for (Map.Entry<String, Candidate> candidateEntry : candidates.entrySet()) {
            candidateEntry.getValue().setMergeQueueNum((int)(candidateEntry.getValue().getMergeQueueNum() * normalizeRatio + 1));
            candidateEntry.getValue().setCandidateNum((int)(candidateEntry.getValue().getCandidateNum() * normalizeRatio + 1));
        }

        return 0;
    }


    public static RetrieverPipeline getRetrievePipeline(Config mergedConfig) {
        return new RetrieverPipeline(loadRetrieveConfig(mergedConfig));
    }

    public static RetrieverPipeline getRetrievePipeline(Config baseConfig, List<Config> configList) {
        Config mergedConfig = ConfigUtils.mergeConfig(baseConfig, configList);
        return getRetrievePipeline(mergedConfig);
    }


    public static Map<String, RetrieveChannel> loadRetrieveConfig(Config config) {
        Map<String, RetrieveChannel> channelsMap = Maps.newHashMap();

        try {
            Config mergedConfig = config.getConfig("retrieve-config");

            // queues
            if (!mergedConfig.hasPath("queues")) {
                // FIXME: throw exception
                LOGGER.error("queues config not exists!");
                return null;
            }

            // retrievers
            if (!mergedConfig.hasPath("configs")) {
                // FIXME: throw exception
                LOGGER.error("retrievers config not exists!");
                return null;
            }

            for (ConfigObject.Entry<String, ConfigValue> it : mergedConfig.getConfig("queues").root().entrySet()) {
                String name = it.getKey();
                Config entryConf = ((ConfigObject)it.getValue()).toConfig();
                if (!entryConf.hasPath("retriever")) {
                    // FIXME: throw exception
                    LOGGER.error("retriever class not exists for queue {}!", name);
                    return null;
                }
                RetrieveChannel channel = new RetrieveChannel(name);
                channel.setRetrieverClass(entryConf.getString("retriever"));
                channelsMap.put(name, channel);
            }

            for (ConfigObject.Entry<String, ConfigValue> it : mergedConfig.getConfig("configs").root().entrySet()) {
                Config entryConf = ((ConfigObject)it.getValue()).toConfig();
                if (channelsMap.containsKey(it.getKey())) {
                    RetrieveConfig retrieveConfig = RetrieveConfig.parseRetrieveConfig(entryConf);
                    RetrieveChannel channel = channelsMap.get(it.getKey());
                    if (channel.getConfig() == null || channel.getConfig().getPriority() < retrieveConfig.getPriority()) {
                        channel.setConfig(retrieveConfig);
                    }
                }
            }

            //
        } catch (Exception e) {
            // error
        }

        return channelsMap;
    }
}
