package com.tzld.piaoquan.recommend.framework.utils;

import org.apache.commons.lang3.tuple.Pair;
import org.apache.http.HttpResponse;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.utils.URIBuilder;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.impl.conn.PoolingHttpClientConnectionManager;
import org.apache.http.protocol.HTTP;
import org.apache.http.util.EntityUtils;

import java.io.IOException;
import java.net.URI;
import java.util.Map;

public class
HttpUtils {
    private static int CONNECT_TIME_OUT = 1000;
    private static int CONNECT_REQUEST_TIME_OUT = 800;
    private static int SOCKET_TIME_OUT = 800;
    private static int MAX_PER_ROUTE = 1000;
    private static int MAX_TOTAL = 1000;

    private static PoolingHttpClientConnectionManager cm = new PoolingHttpClientConnectionManager();
    //set timeout
    private static RequestConfig requestConfig = RequestConfig.custom().setConnectTimeout(
            CONNECT_TIME_OUT).setSocketTimeout(SOCKET_TIME_OUT).setConnectionRequestTimeout(
            CONNECT_REQUEST_TIME_OUT).build();

    // Increase max total connection to MAX_TOTAL
    static {
        cm.setMaxTotal(MAX_TOTAL);

        // Increase default max connection per route to MAX_PER_ROUTE
        cm.setDefaultMaxPerRoute(MAX_PER_ROUTE);

    }

    private static CloseableHttpClient httpClient = HttpClients.custom().setConnectionManager(
            cm).build();

    public static String getContent(String host, String path, Map<String, String> parameters)
            throws IOException {
        try {
            URIBuilder uriBuilder = new URIBuilder().setScheme("http").setHost(host).setPath(path);
            for (Map.Entry<String, String> pEntry : parameters.entrySet()) {
                uriBuilder.setParameter(pEntry.getKey(), pEntry.getValue());
            }
            URI uri = uriBuilder.build();
            HttpGet httpGet = new HttpGet(uri);
            httpGet.setConfig(requestConfig);

            try {
                HttpResponse httpResponse = httpClient.execute(httpGet);
                String result = EntityUtils.toString(httpResponse.getEntity(), HTTP.UTF_8);
                return result;
            } finally {
                httpGet.releaseConnection();
            }

        } catch (Exception e) {
            e.printStackTrace();
            throw new IOException(e);
        }
    }

    public static Pair<Integer, String> getResponse(String host, String path, Map<String, String> parameters)
            throws IOException {
        try {
            URIBuilder uriBuilder = new URIBuilder().setScheme("http").setHost(host).setPath(path);
            for (Map.Entry<String, String> pEntry : parameters.entrySet()) {
                uriBuilder.setParameter(pEntry.getKey(), pEntry.getValue());
            }
            URI uri = uriBuilder.build();
            HttpGet httpGet = new HttpGet(uri);
            httpGet.setConfig(requestConfig);

            try {
                HttpResponse httpResponse = httpClient.execute(httpGet);
                int status = 0;
                if (httpResponse != null && httpResponse.getStatusLine() != null) {
                    status = httpResponse.getStatusLine().getStatusCode();
                }
                String result = null;
                if (status == 200) {
                    result = EntityUtils.toString(httpResponse.getEntity(), HTTP.UTF_8);
                }
                return Pair.of(status, result);
            } finally {
                httpGet.releaseConnection();
            }

        } catch (Exception e) {
            e.printStackTrace();
            throw new IOException(e);
        }
    }

    public static String postContent(String uri, UrlEncodedFormEntity entity) throws IOException {
        try {
            HttpPost httpPost = new HttpPost(uri);
            httpPost.setConfig(requestConfig);
            httpPost.setEntity(entity);
            try {
                HttpResponse httpResponse = httpClient.execute(httpPost);
                String result = EntityUtils.toString(httpResponse.getEntity());
                return result;
            } finally {
                httpPost.releaseConnection();
            }

        } catch (Exception e) {
            e.printStackTrace();
            throw new IOException(e);
        }
    }

}
