package com.tzld.piaoquan.recommend.framework.utils.threadpool;


import java.util.Timer;
import java.util.TimerTask;
import java.util.concurrent.ThreadPoolExecutor;

/**
 *   线程池Helper
 *   可以命名线程池; 保存ThreadPoolExecutor实例, 方便监控线程池信息
 */
public class ThreadPoolHelper {
    private String name;
    private ThreadPoolExecutor threadPoolExecutor;
    private Timer counterTimer;

    ThreadPoolHelper(ThreadPoolExecutor threadPoolExecutor) {
        this.threadPoolExecutor = threadPoolExecutor;
        this.name = "";
    }

    ThreadPoolHelper(ThreadPoolExecutor threadPoolExecutor, String name) {
        this.threadPoolExecutor = threadPoolExecutor;
        this.name = name;
    }

    public String getName() {
        return name;
    }

    public ThreadPoolExecutor getThreadPoolExecutor() {
        return threadPoolExecutor;
    }

    public long getTaskCount() {
        return this.threadPoolExecutor.getTaskCount();
    }

    public long getActiveCount() {
        return this.threadPoolExecutor.getActiveCount();
    }

    public long getCorePoolSize() {
        return this.threadPoolExecutor.getCorePoolSize();
    }

    public long getPoolSize() {
        return this.threadPoolExecutor.getPoolSize();
    }

    public long getMaximumPoolSize() {
        return this.threadPoolExecutor.getMaximumPoolSize();
    }

    public long getLargestPoolSize() {
        return this.threadPoolExecutor.getLargestPoolSize();
    }

    public long getTaskQueueSize() {
        return this.threadPoolExecutor.getQueue().size();
    }

    public void counterThreadPoolInfo(String prefix) {
        // print counters
    }

    /**
     * 开启线程池PerfCounter
     * @param prefix    perf counter name prefix
     * @return          this
     */
    public ThreadPoolHelper enableCounter(final String prefix) {
        if (counterTimer == null) {
            synchronized (this) {
                if (counterTimer == null) {
                    counterTimer = new Timer();
                }
            }
        }
        counterTimer.schedule(new TimerTask() {
            @Override
            public void run() {
                counterThreadPoolInfo(prefix);
            }
        }, 1000, 1000);
        return this;
    }

    public void cancelCounter() {
        if (counterTimer != null) {
            counterTimer.cancel();
            counterTimer.purge();
        }
    }
}
